from unittest import TestCase

from leryan.types import ObjectDict
from leryan.types import FastEnum


class ObjectDictTest(TestCase):

    def test_create(self):
        a = ObjectDict()

        a.attribute = 'value'

        self.assertEqual(a.attribute, 'value')
        self.assertEqual(a['attribute'], 'value')

    def test_create_from_dict(self):
        a = ObjectDict({'attribute': 'value'})

        self.assertEqual(a.attribute, 'value')
        self.assertEqual(a['attribute'], 'value')


class FastEnumTest(TestCase):

    def test_enum(self):
        class TestEnum(FastEnum):

            ATTRIBUTE = 'value'
            OTHER_ATTRIBUTE = 0

        self.assertEqual(TestEnum.ATTRIBUTE, 'value')
        self.assertEqual(TestEnum.OTHER_ATTRIBUTE, 0)

    def test_enum_cannot_reassign(self):
        class TestEnum(FastEnum):

            EXISTING_ATTRIBUTE = 0

        with self.assertRaises(AttributeError):
            TestEnum.EXISTING_ATTRIBUTE = 1

        with self.assertRaises(AttributeError):
            TestEnum.NEW_ATTRIBUTE = 2

    def test_enum_members(self):
        class TestEnum(FastEnum):

            ATTR1 = 0
            attr2 = 0
            Attr3 = 0
            _hidden_attr = 0
            __hidden_attr = 0


        enum_members = TestEnum.members

        self.assertIn('ATTR1', enum_members)
        self.assertIn('attr2', enum_members)
        self.assertIn('Attr3', enum_members)

        self.assertNotIn('_hidden_attr', enum_members)
        self.assertNotIn('__hidden_attr', enum_members)

        self.assertIs(type(enum_members), tuple)
        self.assertEqual(len(enum_members), 3)

    def test_isolation(self):

        class TestEnum1(FastEnum):

            ATTR1 = 1

        class TestEnum2(FastEnum):

            ATTR2 = 2

        self.assertEqual(TestEnum1.ATTR1, 1)
        with self.assertRaises(AttributeError):
            self.assertEqual(TestEnum1.ATTR2, 2)

        with self.assertRaises(AttributeError):
            self.assertEqual(TestEnum2.ATTR1, 1)
        self.assertEqual(TestEnum2.ATTR2, 2)

        enum1_members = TestEnum1.members
        enum2_members = TestEnum2.members

        self.assertNotIn('ATTR2', enum1_members)
        self.assertNotIn('ATTR1', enum2_members)

    def test_enum_values(self):

        class TestEnum(FastEnum):

            ATTR1 = 1
            ATTR2 = 2

        enum_values = TestEnum.values

        self.assertIs(type(enum_values), tuple)
        self.assertEqual(len(enum_values), 2)
        self.assertEqual(TestEnum.ATTR1, 1)
        self.assertEqual(TestEnum.ATTR2, 2)


from future.utils import with_metaclass

class FastEnumMeta(type):

    def __new__(metacls, name, bases, attrs):
        _members = []
        _values = []
        for attr in attrs:
            if not attr.startswith('_'):
                _members.append(attr)
                _values.append(attrs[attr])

        klass = type.__new__(metacls, name, bases, attrs)

        super(metacls, klass).__setattr__('_members', tuple(_members))
        super(metacls, klass).__setattr__('_values', tuple(_values))

        return klass

    def __setattr__(cls, attr, value):
        raise AttributeError("can't set attribute {}".format(attr))

    @property
    def members(cls):
        return cls._members

    @property
    def values(cls):
        return cls._values

class FastEnum(with_metaclass(FastEnumMeta)):

    """
    Fast and simple Enum implementation.

    .. code-block:: python
        class MyEnum(FastEnum):

            MEMBER = 'value'
            OTHER_MEMBER = 0

        MyEnum.MEMBER
        myenum_members = MyEnum.members
    """
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

dependencies = ['future']

setup(
    name='leryan.types',
    version='0.0.12',
    packages=find_packages(exclude=['tests.*']),
    author='Florent Peterschmitt',
    author_email='florent@peterschmitt.fr',
    install_requires=dependencies,
    description='some "types": classes derived from python dict, list...',
    include_package_data=False,
    url='https://github.com/Leryan/leryan.types',
    license='BSD',
    classifiers=[
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6'
    ],
    test_suite='tests',
)
